//	Forward declarations, found in other headers.
class CNstestDoc;

//	Dependent headers, never included standalone....
#ifdef _WIN32
#include "ddeml.h"
#else
#include "nsddeml.h"
#endif // _WIN32
#include "ddeconv.h"

class CDDEObject : public CObject	{
public:
	virtual void Serialize(CArchive& ar);
	DECLARE_SERIAL( CDDEObject )
	
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif // _DEBUG

	CDDEObject(CNstestDoc *pDoc);
	~CDDEObject();
	CNstestDoc *GetDocument()	{ return m_pDoc; }
	
	static CDDEObject *ResolveService(const char *pServiceName);
	static CDDEObject *ResolveService(HSZ hszServiceName);
	static CDDEObject *ResolveConversation(CDDEConversation *pConv);
	
	HDDEDATA ServerConnect(HSZ hszTopic);
	void ServerConnectConfirm(HCONV hConv);
	static void DDEDisconnect(CDDEConversation *pConv);

	//	The DDEML instance identifier.
	static DWORD m_dwidInst;
	
	CString GetServiceName()	{ return m_csServiceName; }
	HSZ GetServiceHSZ()	{ return m_hszServiceName; }

	//	Possible topics.
	enum	{
		m_Activate = 0,
		m_Alert,
		m_BeginProgress,
		m_CancelProgress,
		m_EndProgress,
		m_Exit,
		m_GetWindowInfo,
		m_ListWindows,
		m_MakingProgress,
		m_OpenURL,
		m_ParseAnchor,
		m_QueryURLFile,
		m_QueryViewer,
		m_RegisterProtocol,
		m_RegisterURLEcho,
		m_RegisterViewer,
		m_RegisterWindowChange,
		m_SetProgressRange,
		m_ShowFile,
		m_UnRegisterProtocol,
		m_UnRegisterURLEcho,
		m_UnRegisterViewer,
		m_UnRegisterWindowChange,
		m_URLEcho,
		m_Version,
		m_ViewDocFile,
		m_WindowChange,
		m_MaxTopics
	};
	static int EnumTopic(HSZ hszTopic);
protected:
	//	Dynamic creation.
	CDDEObject();
	
	//	List of currently running CDDEObjects, and our position in the list.
	static CObList m_colRunning;
	POSITION m_rIndex;

	//	Used to generate unique DDE services on a per instance basis.
	static int m_iInstanceCounter;
	CString m_csServiceName;
	HSZ m_hszServiceName;
	
	//	The document owning us.
	CNstestDoc *m_pDoc;
	
	//	The list of DDE object that this service is handling.
	CObList m_colConversations;
	
	static HSZ m_aTopics[m_MaxTopics];
	void LoadTopics();
	void FlushTopics();
	
	//	Client/Server useful methods.
	CDDEConversation *ClientConnect(int iTopic);
	HSZ ClientArguments(const char *pFormat, ...);
	void ServerReturned(HDDEDATA hData, const char *pFormat, ...);
	
	//	Client browser interface.
	static HSZ m_hszBrowser;
public:
	DWORD WWW_Activate(DWORD dwWindowID, DWORD dwFlags = 0ul);
	void WWW_CancelProgress(DWORD dwTransactionID);
	void WWW_Exit();
	void WWW_ListWindows();
	void WWW_Version();
	void WWW_GetWindowInfo(DWORD dwWindowID);
	void WWW_OpenURL(CString csURL, CString csSaveAs, DWORD dwWindowID,
		DWORD dwFlags, CString csPostFormData, CString csPostMIMEType,
		CString csProgressServer);
	void WWW_ParseAnchor(CString csAbsolute, CString csRelative);
	void WWW_QueryURLFile(CString csFileName);
	void WWW_RegisterProtocol(CString csServer, CString csProtocol);
	void WWW_RegisterURLEcho(CString csServer);
	void WWW_RegisterViewer(CString csServer, CString csMIMEType, DWORD dwFlags);
	void WWW_RegisterWindowChange(CString csServer, DWORD dwWindowID);
	void WWW_ShowFile(CString csFileName, CString csMimeType, DWORD dwWindowID,
		CString csURL);
	void WWW_UnRegisterProtocol(CString csServer, CString csProtocol);
	void WWW_UnRegisterURLEcho(CString csServer);
	void WWW_UnRegisterViewer(CString csServer, CString csMIMEType);
	void WWW_UnRegisterWindowChange(CString csServer, DWORD dwWindowID);
	void WWW_WindowChange(DWORD dwWindowID, DWORD dwWindowFlags,
		DWORD dwX, DWORD dwY, DWORD dwCX, DWORD dwCY);
	
private:
	//	Common construction code.
	void CommonConstruction();
};

#define NOTHING ((HDDEDATA)NULL)
#define DDETRUE ((HDDEDATA)TRUE)
#define DDEFALSE ((HDDEDATA)FALSE)
#define DDEFACK ((HDDEDATA)DDE_FACK)

#ifdef _WIN32
HDDEDATA CALLBACK NstestDdeCallBack(UINT type, UINT fmt,
	HCONV hconv, HSZ hsz1, HSZ hsz2, HDDEDATA hData, DWORD dwData1,
	DWORD dwData2);
#else
HDDEDATA CALLBACK _export NstestDdeCallBack(UINT type, UINT fmt,
	HCONV hconv, HSZ hsz1, HSZ hsz2, HDDEDATA hData, DWORD dwData1,
	DWORD dwData2);
#endif // _WIN32

char *SkipToNextArgument(char *pFormat);
char *ExtractArgument(HSZ hszArguments, int iWhichArg);
CString ExplainPoke(HDDEDATA hStatus);
CString ExplainBool(TwoByteBool bFlag);
CString ExplainError();
